
/***********************************************************************/
/* nsock_connect.c -- This contains the functions for requesting       */
/* TCP connections from the nsock parallel socket event library        */
/*                                                                     */
/***********************************************************************/
/*  The nsock parallel socket event library is (C) 2000 Insecure.Com   */
/*  LLC This library is free software; you can redistribute it and/or  */
/*  modify it under the terms of the GNU General Public License as     */
/*  published by the Free Software Foundation; Version 2.  This        */
/*  guarantees your right to use, modify, and redistribute this        */
/*  software under certain conditions.  If this license is             */
/*  unacceptable to you, Insecure.Com LLC may be willing to sell       */
/* alternative licenses (contact fyodor@insecure.org ).                */
/*                                                                     */
/*  If you received these files with a written license agreement       */
/*  stating terms other than the (GPL) terms above, then that          */
/*  alternative license agreement takes precendence over this comment. */
/*                                                                     */
/*  Source is provided to this software because we believe users have  */
/*  a right to know exactly what a program is going to do before they  */
/*  run it.  This also allows you to audit the software for security   */
/*  holes (none have been found so far).                               */
/*                                                                     */
/*  Source code also allows you to port nsock to new platforms, fix    */
/*  bugs, and add new features.  You are highly encouraged to send     */
/*  your changes to Fyodor for possible incorporation into the main    */
/*  distribution.  By sending these changes to Fyodor or one the       */
/*  insecure.org development mailing lists, it is assumed that you     */
/*  are offering Fyodor the unlimited, non-exclusive right to reuse,   */
/*  modify, and relicense the code.  If you wish to specify special    */
/*  license conditions of your contributions, please state them up     */
/*  front.                                                             */
/*                                                                     */
/*  This program is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of     */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  */
/*  General Public License for more details (                          */
/*  http://www.gnu.org/copyleft/gpl.html ).                            */
/*                                                                     */
/***********************************************************************/

/* $Id: nsock_connect.c,v 1.6 2003/07/20 20:40:54 fyodor Exp $ */

#include "nsock.h"
#include "nsock_internal.h"
#include "netutils.h"

#include <sys/types.h>
#include <errno.h>
#include <string.h>

extern struct timeval nsock_tod;

/* Request a TCP connection to another system (by IP address).  The
   in_addr is normal network byte order, but the port number should
   be given in HOST BYTE ORDER */
event_id nsock_connect_tcp(nsock_pool nsp, nsock_iod ms_iod, 
			   nsock_ev_handler handler, int timeout_msecs, 
			   void *userdata, struct in_addr ip, 
			   unsigned short port) {

msiod *nsi = (msiod *) ms_iod;
mspool *ms = (mspool *) nsp;
msevent *nse;
int res;
struct sockaddr_in sock;

assert(nsi->state == NSIOD_STATE_INITIAL || nsi->state == NSIOD_STATE_UNKNOWN);


nse = msevent_new(ms, NSE_TYPE_CONNECT, nsi, timeout_msecs, handler, userdata);
assert(nse);

if (ms->tracelevel > 0)
  nsock_trace(ms, "TCP connection requested to %s:%hi (IOD #%li) EID %li", inet_ntoa(ip), port, nsi->id, nse->id);

/* Now it is time to actually attempt the connection */
 if ((nsi->sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
   perror("Socket troubles");
   nse->event_done = 1; nse->status = NSE_STATUS_ERROR; nse->errnum = errno;
 } else { 
   unblock_socket(nsi->sd);

   sock.sin_family = AF_INET;
   sock.sin_addr = ip;
   sock.sin_port = htons(port);

   nsi->peer = ip;
   nsi->peerport = port;

   if ((res = connect(nsi->sd, (struct sockaddr *) &sock,
		      sizeof(struct sockaddr_in))) != -1) {
     nse->event_done = 1; nse->status = NSE_STATUS_SUCCESS;
   } else if (errno != EINPROGRESS && errno != EAGAIN) {
     nse->event_done = 1; nse->status = NSE_STATUS_ERROR; nse->errnum = errno;
   }
 }

 nsp_add_event(ms, nse);

 return nse->id;
}

/* Request a UDP "connection" to another system (by IP address).  The
   in_addr is normal network byte order, but the port number should be
   given in HOST BYTE ORDER.  Since this is UDP, no packets are
   actually sent.  The destination IP and port are just associated
   with the nsiod (an actual OS connect() call is made).  You can then
   use the normal nsock write calls on the socket.  There is no
   timeout since this call always calls your callback at the next
   opportunity.  The advantages to having a connected UDP socket (as
   opposed to just specifying an address with sendto() are that we can
   now use a consistent set of write/read calls for TCP/UDP, received
   packets from the non-partner are automatically dropped by the OS,
   and the OS can provide asynchronous errors (see Unix Network
   Programming pp224) */
event_id nsock_connect_udp(nsock_pool nsp, nsock_iod nsiod, 
			   nsock_ev_handler handler, void *userdata, 
			   struct in_addr ip, unsigned short port) {

msiod *nsi = (msiod *) nsiod;
mspool *ms = (mspool *) nsp;
msevent *nse;
int res;
struct sockaddr_in sock;

assert(nsi->state == NSIOD_STATE_INITIAL || nsi->state == NSIOD_STATE_UNKNOWN);

nse = msevent_new(ms, NSE_TYPE_CONNECT, nsi, -1, handler, userdata);
assert(nse);

if (ms->tracelevel > 0)
  nsock_trace(ms, "UDP connection requested to %s:%hi (IOD #%li) EID %li", inet_ntoa(ip), port, nsi->id, nse->id);

/* Now it is time to actually attempt the connection */
 if ((nsi->sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
   perror("Socket troubles");
   nse->event_done = 1; nse->status = NSE_STATUS_ERROR; nse->errnum = errno;
 } else { 
   unblock_socket(nsi->sd);

   sock.sin_family = AF_INET;
   sock.sin_addr = ip;
   sock.sin_port = htons(port);

   nsi->peer = ip;
   nsi->peerport = port;

   if ((res = connect(nsi->sd, (struct sockaddr *) &sock,
		      sizeof(struct sockaddr_in))) != -1) {
     nse->event_done = 1; nse->status = NSE_STATUS_SUCCESS;
   } else {
     nse->event_done = 1; nse->status = NSE_STATUS_ERROR; nse->errnum = errno;
   }
 }

 nsp_add_event(ms, nse);

 return nse->id;
}

/* Returns that host/port/protocol information for the last
   communication (or comm. attempt) this nsi has been involved with.
   By "involved" with I mean interactions like establishing (or trying
   to) a connection or sending a UDP datagram through an unconnected
   nsock_iod.  Protocl is IPPROTO_TCP or IPPROTO_UDP.  Pass NULL for
   information you do not need.  If ANY of the information you
   requested is not available, 0 will be returned and the unavailable
   addy & port are zeroed (but note that addy can
   legitimately be 0).  If protocol is requested but not available, it
   will be set to -1 (and 0 returned).  The pointers you pass in must
   be NULL or point to allocated address space */

int nsi_getlastcommunicationinfo(nsock_iod ms_iod, int *protocol, 
				 struct in_addr *localaddy, 
				 unsigned short *localport, 
				 struct in_addr *remoteaddy, 
				 unsigned short *remoteport) {
  msiod *nsi = (msiod *) ms_iod;
  int ret = 1;
  struct sockaddr_in sock;
  int socklen = sizeof(struct sockaddr_in);
  int res;

  if (nsi->peerport) { /* peer and peerport are valid */
    if (remoteaddy) *remoteaddy = nsi->peer;
    if (remoteport) *remoteport = nsi->peerport;
    if (protocol) { 
      *protocol = nsi->lastproto; 
      if (*protocol == -1) res = 0; 
    }
    if (nsi->sd >= 0) {
      res = getsockname(nsi->sd, (struct sockaddr *) &sock, &socklen);
      if (res == -1) {
	if (localaddy) { memset(localaddy, 0, sizeof(*localaddy)); ret = 0;	}
	if (localport) { *localport = 0; ret = 0;  }
      } else {
	if (localaddy) { *localaddy = sock.sin_addr; }
	if (localport) { *localport = ntohs(sock.sin_port); }
      }
    } else {
      if (localaddy) { memset(localaddy, 0, sizeof(*localaddy)); ret = 0;	}
      if (localport) { *localport = 0; ret = 0;  }
    }
  } else {
    if (localaddy || localport || remoteaddy || remoteport || protocol)
      ret = 0;
    if (remoteaddy) memset(remoteaddy, 0, sizeof(*remoteaddy));
    if (remoteport) *remoteport = 0;
    /* if there is no peer, there won't be any local info */
    if (localaddy) memset(localaddy, 0, sizeof(*localaddy));
    if (localport) *localport = 0;
    if (protocol) *protocol = -1;
  }
  return ret;
}
